@@ -8,7 +8,7 @@
 #
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 
 '''Command line interface for *didjvu*'''
@@ -64,12 +64,14 @@ class intact(object):
 class ArgumentParser(argparse.ArgumentParser):
 
     def __init__(self, methods, default_method):
-        argparse.ArgumentParser.__init__(self)
+        argparse.ArgumentParser.__init__(self, formatter_class=argparse.RawDescriptionHelpFormatter)
         self.add_argument('--version', action='version', version='%(prog)s ' + __version__, help='show version information and exit')
         p_separate = self.add_subparser('separate', help='generate masks for images')
         p_encode = self.add_subparser('encode', help='convert images to single-page DjVu documents')
         p_bundle = self.add_subparser('bundle', help='convert images to bundled multi-page DjVu document')
-        for p in p_encode, p_separate, p_bundle:
+        epilog = []
+        for p in p_separate, p_encode, p_bundle:
+            epilog += ['%s --help' % p.prog]
             p.add_argument('-o', '--output', metavar='FILE', help='output filename')
             if p is p_bundle:
                 p.add_argument('--pageid-template', metavar='TEMPLATE', default='{base-ext}.djvu', help='naming scheme for page identifiers')
@@ -91,7 +93,7 @@ class ArgumentParser(argparse.ArgumentParser):
                     p.add_argument('--%s-crcb' % layer, choices='normal half full none'.split(), help='chrominance encoding for %s' % layer_name)
                     p.add_argument('--%s-subsample' % layer, type=subsample_type, metavar='N', help='subsample ratio for %s' % layer_name)
                 p.add_argument('--fg-bg-defaults', help=argparse.SUPPRESS, action='store_const', const=1)
-            if p is not p_encode:
+            if p is not p_separate:
                 p.add_argument('-d', '--dpi', type=dpi_type, metavar='N', help='image resolution')
             if p is p_bundle:
                 p.add_argument('-p', '--pages-per-dict', type=int, metavar='N', help='how many pages to compress in one pass')
@@ -106,9 +108,11 @@ class ArgumentParser(argparse.ArgumentParser):
                 pages_per_dict=1,
                 dpi=djvu.DPI_DEFAULT,
                 fg_slices=intact([100]), fg_crcb=intact('full'), fg_subsample=intact(6),
-                bg_slices=intact([72, 82, 88, 95]), bg_crcb=intact('normal'), bg_subsample=intact(3),
+                bg_slices=intact([74, 84, 90, 97]), bg_crcb=intact('normal'), bg_subsample=intact(3),
                 verbosity=[None],
             )
+        self.epilog = 'more help:\n  ' + '\n  '.join(epilog)
+        self.__methods = methods
 
     def add_subparser(self, name, **kwargs):
         try:
@@ -141,6 +145,7 @@ class ArgumentParser(argparse.ArgumentParser):
         if o.pages_per_dict <= 1:
             o.pages_per_dict = 1
         action = getattr(actions, vars(o).pop('_action_'))
+        o.method = self.__methods[o.method]
         return action(o)
 
 __all__ = ['ArgumentParser']
