@@ -9,13 +9,14 @@
 #
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 
 from __future__ import with_statement
 
 import os
 import re
+import logging
 import string
 import sys
 
@@ -25,9 +26,102 @@ from . import filetype
 from . import gamera_extra as gamera
 from . import ipc
 from . import temporary
-from . import tinylog
 
-formatter = string.Formatter()
+logger = None
+
+try:
+    from string import Formatter
+except ImportError:
+    # This is very rough, incomplete backport of string.Formatter class.
+    # It's provided here to allow basic functionality of didjvu even with
+    # Python 2.5.
+
+    import warnings
+    warnings.warn(RuntimeWarning('Python %d.%d is not supported, please use Python >= 2.6' % sys.version_info[:2]), stacklevel=999)
+
+    class Formatter():
+
+        _split = re.compile('''
+            ( [{][{] | [}][}] | [{][^}]*[}] )
+        ''', re.VERBOSE).split
+
+        @staticmethod
+        def _not_implemented():
+            raise NotImplementedError('Please use Python >= 2.6')
+
+        def format(self, format_string, *args, **kwargs):
+            return self.vformat(format_string, args, kwargs)
+
+        def vformat(self, format_string, args, kwargs):
+            result = []
+            for literal_text, field_name, format_spec, conversion in self.parse(format_string):
+                if conversion is not None:
+                    self._not_implemented()
+                if literal_text:
+                    result += [literal_text]
+                if field_name is not None:
+                    obj, _ = self.get_field(field_name, args, kwargs)
+                    obj = self.convert_field(obj, conversion)
+                    result += [self.format_field(obj, format_spec)]
+            return ''.join(result)
+
+        def convert_field(self, value, conversion):
+            if conversion is None:
+                return value
+            else:
+                self._not_implemented()
+
+        def get_value(self, field_name, args, kwargs):
+            if args:
+                self._not_implemented()
+            return kwargs[field_name]
+
+        def get_field(self, field_name, args, kwargs):
+            return self.get_value(field_name, args, kwargs), field_name
+
+        def format_field(self, value, format_spec):
+            if format_spec is not None:
+                self._not_implemented()
+            return str(value)
+
+        def parse(self, format_string):
+            for token in self._split(format_string):
+                if token in ('{{', '}}'):
+                    yield token[0], None, None, None
+                elif token[:1] + token[-1:] == '{}':
+                    if (':' in token) or ('!' in token) or ('.' in token):
+                        self._not_implemented()
+                    yield '', token[1:-1], None, None
+                elif token:
+                    yield token, None, None, None
+
+        def get_value(self, key, args, kwargs):
+            if isinstance(key, (int, long)):
+                return args[key]
+            else:
+                return kwargs[key]
+
+formatter = Formatter()
+del Formatter
+
+def setup_logging():
+    global logger
+    logger = logging.getLogger('didjvu.main')
+    ipc_logger = logging.getLogger('didjvu.ipc')
+    logging.NOSY = (logging.INFO + logging.DEBUG) // 2
+    def nosy(msg, *args, **kwargs):
+        logger.log(logging.NOSY, msg, *args, **kwargs)
+    logger.nosy = nosy
+    # Main handler:
+    handler = logging.StreamHandler()
+    formatter = logging.Formatter('%(message)s')
+    handler.setFormatter(formatter)
+    logger.addHandler(handler)
+    # IPC handler:
+    handler = logging.StreamHandler()
+    formatter = logging.Formatter('+ %(message)s')
+    handler.setFormatter(formatter)
+    ipc_logger.addHandler(handler)
 
 def check_tty():
     if sys.stdout.isatty():
@@ -192,7 +286,7 @@ def expand_template(template, name, page):
         if not isinstance(base_value, int):
             continue
         d[var] = d[base_var] + offset
-    return formatter.format(template, **d)
+    return formatter.vformat(template, (), d)
 
 _pageid_chars = re.compile('^[A-Za-z0-9_+.-]+$').match
 
@@ -204,10 +298,10 @@ def check_pageid_sanity(pageid):
     if '..' in pageid:
         raise ValueError('Pageid cannot contain two consecutive dots.')
     assert pageid == os.path.basename(pageid)
-    if pageid.endswith('.djvu') or pageid.endswith('.djv'):
+    if pageid.endswith('.djvu'):
         return pageid
     else:
-        raise ValueError('Pageid must end with the .djvu or the .djv extension.')
+        raise ValueError('Pageid must end with the .djvu extension.')
 
 def replace_ext(filename, ext):
     return '%s.%s' % (os.path.splitext(filename)[0], ext)
@@ -223,16 +317,24 @@ class main():
         '%(bytes_in)d bytes in, %(bytes_out)d bytes out'
 
     def __init__(self):
-        parser = cli.ArgumentParser(gamera.methods, default_method=gamera.methods['djvu'])
+        setup_logging()
+        parser = cli.ArgumentParser(gamera.methods, default_method='djvu')
         parser.parse_args(actions=self)
 
     def check_common(self, o):
-        ipc.DEBUG = o.verbosity >= 2
         if len(o.masks) == 0:
             o.masks = [None for x in o.input]
         elif len(o.masks) != len(o.input):
             raise ValueError('%d input images != %d masks' % (len(o.input), len(o.masks)))
-        self.log = tinylog.Log(o.verbosity)
+        ipc_logger = logging.getLogger('didjvu.ipc')
+        assert logger is not None
+        log_level = {
+            0: logging.WARNING,
+            1: logging.INFO,
+            2: logging.NOSY,
+        }.get(o.verbosity, logging.DEBUG)
+        logger.setLevel(log_level)
+        ipc_logger.setLevel(log_level)
         gamera.init()
 
     def check_multi_output(self, o):
@@ -272,11 +374,11 @@ class main():
 
     def encode_one(self, o, image_filename, mask_filename, output):
         bytes_in = os.path.getsize(image_filename)
-        print >>self.log(1), '%s:' % image_filename
+        logger.info('%s:' % image_filename)
         ftype = filetype.check(image_filename)
         if ftype.like(filetype.djvu):
             if ftype.like(filetype.djvu_single):
-                print >>self.log(1), '- copying DjVu as is'
+                logger.info('- copying DjVu as is')
                 with open(image_filename, 'rb') as djvu_file:
                     copy_file(djvu_file, output)
             else:
@@ -284,12 +386,12 @@ class main():
                 # consist of. If it's only one, continue.
                 raise NotImplementedError("I don't know what to do with this file")
             return
-        print >>self.log(1), '- reading image'
+        logger.info('- reading image')
         image = gamera.load_image(image_filename)
         width, height = image.ncols, image.nrows
-        print >>self.log(2), '- image size: %d x %d' % (width, height)
+        logger.nosy('- image size: %d x %d', width, height)
         mask = generate_mask(mask_filename, image, o.method)
-        print >>self.log(1), '- converting to DjVu'
+        logger.info('- converting to DjVu')
         djvu_doc = image_to_djvu(width, height, image, mask, options=o)
         djvu_file = djvu_doc.save()
         try:
@@ -299,23 +401,23 @@ class main():
         bits_per_pixel = 8.0 * bytes_out / (width * height)
         ratio = 1.0 * bytes_in / bytes_out
         percent_saved = (1.0 * bytes_in - bytes_out) * 100 / bytes_in;
-        print >>self.log(2), ('- %s' % self.compression_info_template) % locals()
+        logger.info('- %s', self.compression_info_template % locals())
 
     def separate_one(self, o, image_filename, output):
         bytes_in = os.path.getsize(image_filename)
-        print >>self.log(1), '%s:' % image_filename
+        logger.info('%s:', image_filename)
         ftype = filetype.check(image_filename)
         if ftype.like(filetype.djvu):
             # TODO: Figure out if how many pages the document consist of.
             # If it's only one, extract the existing mask.
             raise NotImplementedError("I don't know what to do with this file")
-        print >>self.log(1), '- reading image'
+        logger.info('- reading image')
         image = gamera.load_image(image_filename)
         width, height = image.ncols, image.nrows
-        print >>self.log(2), '- image size: %d x %d' % (width, height)
-        print >>self.log(1), '- thresholding'
+        logger.nosy('- image size: %d x %d' % (width, height))
+        logger.info('- thresholding')
         mask = generate_mask(None, image, o.method)
-        print >>self.log(1), '- saving'
+        logger.info('- saving')
         if output is not sys.stdout:
             # A real file
             mask.save_PNG(output.name)
@@ -353,7 +455,7 @@ class main():
                 component_filenames += os.path.join(tmpdir, pageid),
                 with open(component_filenames[-1], 'wb') as component:
                     self.encode_one(o, input, mask, component)
-            print >>self.log(1), 'bundling'
+            logger.info('bundling')
             djvu_file = djvu.bundle_djvu(*component_filenames)
             try:
                 bytes_out = copy_file(djvu_file, output)
@@ -362,7 +464,7 @@ class main():
         bits_per_pixel = float('nan') # FIXME!
         ratio = 1.0 * bytes_in / bytes_out
         percent_saved = (1.0 * bytes_in - bytes_out) * 100 / bytes_in;
-        print >>self.log(2), self.compression_info_template % locals()
+        logger.nosy(self.compression_info_template % locals())
 
     def bundle_complex(self, o):
         [output] = o.output
@@ -377,25 +479,25 @@ class main():
                 page.pageid = expand_template(o.pageid_template, image_filename, pageno)
                 check_pageid_sanity(page.pageid)
                 # TODO: Check for filename conflicts.
-                print >>self.log(1), '%s:' % image_filename
+                logger.info('%s:', image_filename)
                 ftype = filetype.check(image_filename)
                 if ftype.like(filetype.djvu):
                     # TODO: Allow to merge existing documents (even multi-page ones).
                     raise NotImplementedError("I don't know what to do with this file")
-                print >>self.log(1), '- reading image'
+                logger.info('- reading image')
                 image = gamera.load_image(image_filename)
                 width, height = image.ncols, image.nrows
                 pixels += width * height
-                print >>self.log(2), '- image size: %d x %d' % (width, height)
+                logger.nosy('- image size: %d x %d', width, height)
                 mask = generate_mask(mask_filename, image, o.method)
-                print >>self.log(1), '- converting to DjVu'
+                logger.info('- converting to DjVu')
                 page.djvu = image_to_djvu(width, height, image, mask, options=o)
                 image = mask = None
                 page.sjbz = djvu.Multichunk(width, height, o.dpi, sjbz=page.djvu['sjbz'])
                 page.sjbz_symlink = os.path.join(minidjvu_in_dir, page.pageid)
                 os.symlink(page.sjbz.save().name, page.sjbz_symlink)
             with temporary.directory() as minidjvu_out_dir:
-                print >>self.log(1), 'creating shared dictionaries'
+                logger.info('creating shared dictionaries')
                 def chdir():
                     os.chdir(minidjvu_out_dir)
                 arguments = ['minidjvu',
@@ -414,6 +516,7 @@ class main():
                     if pageno % o.pages_per_dict == 0:
                         iff_name = replace_ext(page_info[pageno].pageid, 'iff')
                         iff_name = os.path.join(minidjvu_out_dir, iff_name)
+                        component_filenames += [iff_name]
                     sjbz_name = os.path.join(minidjvu_out_dir, page.pageid)
                     component_filenames += [sjbz_name]
                     page.djvu['sjbz'] = sjbz_name
@@ -422,7 +525,7 @@ class main():
                     page.djvu_symlink = os.path.join(minidjvu_out_dir, page.pageid)
                     os.unlink(page.djvu_symlink)
                     os.symlink(page.djvu.name, page.djvu_symlink)
-                print >>self.log(1), 'bundling'
+                logger.info('bundling')
                 djvu_file = djvu.bundle_djvu(*component_filenames)
                 try:
                     bytes_out = copy_file(djvu_file, output)
@@ -431,6 +534,6 @@ class main():
         bits_per_pixel = 8.0 * bytes_out / pixels
         ratio = 1.0 * bytes_in / bytes_out
         percent_saved = (1.0 * bytes_in - bytes_out) * 100 / bytes_in;
-        print >>self.log(2), self.compression_info_template % locals()
+        logger.nosy('%s', self.compression_info_template % locals())
 
 # vim:ts=4 sw=4 et
