@@ -8,7 +8,7 @@
 #
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 
 '''Bridge to the Gamera framework'''
@@ -16,9 +16,10 @@
 import os
 import re
 import sys
+import warnings
 
 from gamera.core import load_image as _load_image
-from gamera.core import init_gamera as init
+from gamera.core import init_gamera as _init
 from gamera.core import Image, RGB, GREYSCALE, ONEBIT, Dim, RGBPixel
 from gamera.plugins.pil_io import from_pil as _from_pil
 
@@ -28,11 +29,14 @@ except ImportError:
     load_image = _load_image
 else:
     def load_image(filename):
-        pil_image = PIL.open(filename)
-        if pil_image.mode == '1':
-            # Gamera supports importing only 8-bit and RGB from PIL:
-            pil_image = pil_image.convert('L')
         # TODO: Try to load image without PIL, even if it is imported.
+        pil_image = PIL.open(filename)
+        # Gamera supports importing only 8-bit and RGB from PIL:
+        if pil_image.mode == '1':
+            pil_image = pil_image.convert('L')
+        elif pil_image.mode not in ('RGB', 'L'):
+            pil_image = pil_image.convert('RGB')
+        assert pil_image.mode in ('RGB', 'L')
         return _from_pil(pil_image)
 
 def colorspace_wrapper(plugin):
@@ -77,4 +81,47 @@ methods = _load_methods()
 def to_pil_1bpp(image):
     return image.to_greyscale().to_pil()
 
+def _decref(o):
+    '''
+    Forcibly set refcount of the object to 1.
+    '''
+    import ctypes
+    libc = ctypes.cdll.LoadLibrary(None)
+    memmove = libc.memmove
+    memmove.argtypes = [ctypes.py_object, ctypes.py_object, ctypes.c_size_t]
+    memmove.restype = ctypes.py_object
+    return memmove(o, None, 0)
+
+def _monkeypatch_to_raw_string():
+    '''
+    Monkey-patch to _to_raw_string plugin function to return objects with
+    correct refcounts.
+    '''
+    from gamera.plugins import _string_io
+    old_to_raw_string = _string_io._to_raw_string
+    def fixed_to_raw_string(*args, **kwargs):
+        o = old_to_raw_string(*args, **kwargs)
+        assert sys.getrefcount(o) == 3
+        o = _decref(o)
+        assert sys.getrefcount(o) == 2
+        return o
+    _string_io._to_raw_string = fixed_to_raw_string
+    Image._to_raw_string = fixed_to_raw_string
+
+def init():
+    result = _init()
+    test_image = Image((0, 0), (5, 5), RGB)
+    test_string = test_image._to_raw_string()
+    refcount = sys.getrefcount(test_string)
+    if refcount == 3:
+        # See: http://tech.groups.yahoo.com/group/gamera-devel/message/2068
+        warnings.warn(RuntimeWarning('Working around memory leak in the Gamera library'), stacklevel=2)
+        _monkeypatch_to_raw_string()
+        fixed_test_string = test_image._to_raw_string()
+        assert sys.getrefcount(fixed_test_string) == 2
+        assert test_string == fixed_test_string
+    else:
+        assert refcount == 2
+    return result
+
 # vim:ts=4 sw=4 et
