@@ -8,18 +8,24 @@
 #
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 
+import logging
 import os
 import re
-import sys
 import signal
 import subprocess
+import sys
 
 from subprocess import CalledProcessError
 
-DEBUG = False
+logger = logging.getLogger('didjvu.ipc')
+
+# Protect from scanadf[0] and possibly other brain-dead software that set
+# SIGCHLD to SIG_IGN.
+# [0] http://bugs.debian.org/596232
+signal.signal(signal.SIGCHLD, signal.SIG_DFL)
 
 def get_signal_names():
     _signame_pattern = re.compile('^SIG[A-Z0-9]*$')
@@ -55,35 +61,41 @@ del get_signal_names
 PIPE = subprocess.PIPE
 from subprocess import CalledProcessError
 
-def shell_escape(s, safe=re.compile('^[a-zA-Z0-9_+/=.,-]+$').match):
+def shell_escape(s, safe=re.compile('^[a-zA-Z0-9_+/=.,:-]+$').match):
     if safe(s):
         return s
     return "'%s'" % s.replace("'", r"'\''")
 
 class Subprocess(subprocess.Popen):
 
-    def __init__(self, *args, **kwargs):
-        env_override = kwargs.pop('env', {})
-        if env_override.get('LC_ALL', '') is None:
-            # Reset all locale variables
-            env = dict(
+    @classmethod
+    def override_env(cls, override):
+        env = os.environ
+        # We'd like to:
+        # - preserve LC_CTYPE (which is required by some DjVuLibre tools),
+        # - but reset all other locale settings (which tend to break things).
+        lc_ctype = env.get('LC_ALL') or env.get('LC_CTYPE') or env.get('LANG')
+        env = dict(
                 (k, v)
-                for k, v in os.environ.iteritems()
+                for k, v in env.iteritems()
                 if not (k.startswith('LC_') or k in ('LANG', 'LANGUAGES'))
-            )
-            del env_override['LC_ALL']
-        else:
-            env = dict(os.environ)
-        env.update(env_override)
-        kwargs['env'] = env
+        )
+        if lc_ctype:
+            env['LC_CTYPE'] = lc_ctype
+        if override:
+            env.update(override)
+        return env
+
+    def __init__(self, *args, **kwargs):
+        kwargs['env'] = self.override_env(kwargs.get('env'))
         if os.name == 'posix':
             kwargs.update(close_fds=True)
         try:
             commandline = kwargs['args']
         except KeyError:
             commandline = args[0]
-        if DEBUG:
-            print >>sys.stderr, '+', ' '.join(shell_escape(s) for s in commandline)
+        if logger.isEnabledFor(logging.DEBUG):
+            logger.debug(' '.join(shell_escape(s) for s in commandline))
         self.__command = commandline[0]
         subprocess.Popen.__init__(self, *args, **kwargs)
 
