@@ -8,7 +8,7 @@
 #
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 
 import contextlib
@@ -23,7 +23,7 @@ wrapper = tempfile._TemporaryFileWrapper
 @contextlib.contextmanager
 def directory(*args, **kwargs):
     kwargs = dict(kwargs)
-    kwargs.setdefault('prefix', 'didjvu')
+    kwargs.setdefault('prefix', 'didjvu.')
     tmpdir = tempfile.mkdtemp(*args, **kwargs)
     try:
         yield tmpdir
